# SPDX-FileCopyrightText: 2019,2020 Freemelt AB
#
# SPDX-License-Identifier: Apache-2.0

"""
Copyright 2019,2020 Freemelt AB

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

"""@Transformations docstring
A collection of methods to be used when dealing with transformations of lists of beam segments.

"""


def translate(obpdata, V):
    translated = []
    for segment in obpdata:
        translated.append(segment.translate(V))
    return translated


def rotate(obpdata, theta):
    rotated = []
    for segment in obpdata:
        rotated.append(segment.rotate(theta))
    return rotated


def scale(obpdata, factor):
    scaled = []
    for segment in obpdata:
        scaled.append(segment.scale(factor))
    return scaled
